/**
 * Class for representing edges in graphs. An edge is defined by its own id and the ids of the two knots it
 * connects. Furthermore, an arbitrary integer value is associated with each edge. This value is used for data
 * to be visualized.
 *
 * @author Stefan Daschek
 */
public class Edge {

  /** Id of the edge. */
  public int id;

  /** Id of the first knot. */
  public int from;

  /** Id of the second knot. */
  public int to;

  /** Value associated with edge. */
  public int val;

  /**
   * Default constructor.
   *
   * @param id   id of the edge
   * @param from id of first knot
   * @param to   id of second knot
   * @param val  value associated with edge
   */
  public Edge( int id, int from, int to, int val ) {
    this.id   = id;
    this.from = from;
    this.to   = to;
    this.val  = val;
  }
}
