/**
 * Class for creating NetVis userinterface.
 *
 * @author Stefan Daschek
 */
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;


public class NetVisUI implements ItemListener, ActionListener
{
  final static String OPT_2D_LOAD     = "2d Auslastung";
  final static String OPT_3D_LOAD     = "3d Auslastung";
  final static String OPT_3D_CAPACITY = "3d Kapazitt";

  static NetVisUI app;

  /** Data to be visualized. */
  NetData netData;

  /** Visualization class. */
  NetVis nv;
  
  public NetVisUIDesktopFrame desktop;
  JInternalFrame graph;
  
  public void start()
  {
    nv = new NetVis();

    //Desktop erzeugen
    desktop = new NetVisUIDesktopFrame( this );
    desktop.setLocation(100, 60);
    desktop.setSize(800, 600);
    desktop.setVisible(true);
    desktop.addChild( new NetVisUIChildFrame1( "Kanten (2D)", this), 10, 10 );
    desktop.addChild( new NetVisUIChildFrame2( "Knoten (3D)", this), 10, 150 );
    graph = desktop.addGraf( new NetVisUIChildFrame3( "Visualisierung", nv, this), 180, 10 );
    
    try {
      netData = new NetData( "data/test-1" );
      nv.setNetData( netData );
      desktop.repaint();
    }
    catch( Exception ex ) {
      System.err.println( "Exception: " + ex.getMessage() );
    }
  }

  /**
   * Main method. Create userinterface and initialize application.
   */
  public static void main(String[] args)
  {
    app = new NetVisUI();
    app.start();
  }


  public void itemStateChanged(ItemEvent e)
  {
    JCheckBox cb = (JCheckBox)e.getSource();
    int change = e.getStateChange();
    String s = cb.getName();
    
    if( s == OPT_3D_LOAD )
      nv.setShow3dLoad( change == e.SELECTED );
    else if( s == OPT_3D_CAPACITY )
      nv.setShow3dCapacity( change == e.SELECTED );
    else if( s == OPT_2D_LOAD )
      nv.setShow2dPies( change == e.SELECTED );
  }

  public void actionPerformed( ActionEvent e )
  {
    String cmd = e.getActionCommand();
    
    if (cmd.equals("Beenden"))
    {
      desktop.setVisible(false);
      desktop.dispose();
      System.exit(0);
    }
    else if (cmd.equals("Testdatensatz 1") || cmd.equals("Testdatensatz 2"))
    {
      graph.dispose();
      nv = new NetVis();
      NetData nd  = null;
      try
      {
        if( cmd.equals( "Testdatensatz 1" ) )
          nd = new NetData( "data/test-1" );
        else
          nd = new NetData( "data/test-2" );
      }
      catch( Exception ex )
      {
        System.err.println( "Exception: " + ex.getMessage() );
      }
      graph = desktop.addGraf( new NetVisUIChildFrame3( "Visualisierung", nv, this), 180, 10 );
      nv.setNetData( nd );
      nv.setShow3dLoad( desktop.cbShow3dLoad.isSelected() );
      nv.setShow3dCapacity( desktop.cbShow3dCapacity.isSelected() );
      nv.setShow2dPies( desktop.cbShow2dLoad.isSelected() );
      desktop.repaint();
    }
    else if( cmd.equals( "Reset" ) )
      nv.reset3d();
  }
}



class NetVisUIDesktopFrame extends JFrame 
{
  private JDesktopPane desk;
  public String[][] coord = new String[100][10];
  JInternalFrame graph;
  NetVisUI ui;
  public JCheckBox cbShow2dLoad;
  public JCheckBox cbShow3dLoad;
  public JCheckBox cbShow3dCapacity;


  /**
   * Default constructor.
   */
  public NetVisUIDesktopFrame( NetVisUI ui )
  {
    super("NetVis 0.5");
    this.desk = new JDesktopPane();
    this.ui = ui;
    desk.setDesktopManager(new DefaultDesktopManager());
    setContentPane(desk);
    addWindowListener(new WindowClosingAdapter(true));
    addComponentListener( new ComponentAdapter() {
        public void componentResized( ComponentEvent e ) {
          if( graph != null )
            graph.setSize( desk.getSize().width - 190, desk.getSize().height - 20);
        }
      });
    JMenuBar menubar = new JMenuBar();
    menubar.add(createFileMenu());
//     menubar.add(createScreenMenu());
    setJMenuBar(menubar);
    
  }
	


	//---Private Methoden----------
	private JMenu createFileMenu()
	{
		JMenu ret = new JMenu("Datei");
		ret.setMnemonic('D');
		JMenuItem mi;
		//ffnen
		mi = new JMenuItem("Testdatensatz 1", '1');
		mi.addActionListener( (ActionListener)ui );
		ret.add(mi);
		mi = new JMenuItem("Testdatensatz 2", '2');
		mi.addActionListener( (ActionListener)ui );
		ret.add(mi);
		//Seperator
		ret.addSeparator();
		//Beenden
		mi = new JMenuItem("Beenden", 'e');
		mi.addActionListener( (ActionListener)ui );
		ret.add(mi);
		return ret;
	}
	
	
	private void setCtrlAccelerator(JMenuItem mi, char acc)
	{
		KeyStroke ks = KeyStroke.getKeyStroke(acc, Event.CTRL_MASK);
		mi.setAccelerator(ks);
	}

	public void addChild(JInternalFrame child, int x, int y)
	{
          child.setLocation(x, y);
          child.setClosable( false );
          child.setIconifiable( false );
          child.setMaximizable( false );
          child.setResizable( false );
          child.setSize(160, 130);
          desk.add(child);
          child.setVisible(true);
	}
	
	public JInternalFrame addGraf(JInternalFrame child, int x, int y)
	{
          child.setLocation(x, y);
          child.setClosable( false );
          child.setIconifiable( false );
          child.setMaximizable( false );
          child.setResizable( false );
          child.setSize( desk.getSize().width - 190, desk.getSize().height - 20);
          desk.add(child);
          child.setVisible(true);
          graph = child;
          return child;
	}
}

// Childframe1 Klasse fr Optionen
class NetVisUIChildFrame1 extends JInternalFrame
{
  public NetVisUIChildFrame1( String title, NetVisUI ui )
  {
    super(title, true, true);
    setIconifiable(false);
    setMaximizable(false);
    setClosable(false);
    setResizable(false);
    setDefaultCloseOperation(HIDE_ON_CLOSE);
    setBackground(Color.lightGray);
    JPanel panel = new JPanel();
    panel.setLayout(new GridLayout(3, 1));
    
    ui.desktop.cbShow2dLoad = new JCheckBox("Auslastung", false);
    ui.desktop.cbShow2dLoad.setName( ui.OPT_2D_LOAD );
    ui.desktop.cbShow2dLoad.addItemListener( (ItemListener)ui );
    panel.add(ui.desktop.cbShow2dLoad);
    
    JCheckBox cb = new JCheckBox("Kapazitt", false);
    cb.setEnabled( false );
    panel.add(cb);
    
    cb = new JCheckBox("Animation", false);
    cb.setEnabled( false );
    panel.add(cb);
    
    getContentPane().add(panel, BorderLayout.CENTER);
  }
	
}

//NetVisUIChildFrame2 Klasse fr 3D-Optionen
class NetVisUIChildFrame2 extends JInternalFrame
{
  public NetVisUIChildFrame2( String title, NetVisUI ui )
  {
    super(title, true, true);
    setIconifiable(false);
    setMaximizable(false);
    setClosable(false);
    setResizable(false);
    setDefaultCloseOperation(HIDE_ON_CLOSE);
    setBackground(Color.lightGray);
    
    JPanel panel = new JPanel();
    panel.setLayout(new GridLayout(3, 1));

    
    ui.desktop.cbShow3dLoad = new JCheckBox( "Auslastung", false );
    ui.desktop.cbShow3dLoad.setName( ui.OPT_3D_LOAD );
    ui.desktop.cbShow3dLoad.addItemListener( (ItemListener)ui );
    panel.add(ui.desktop.cbShow3dLoad);

    ui.desktop.cbShow3dCapacity = new JCheckBox( "Kapazitt", false );
    ui.desktop.cbShow3dCapacity.setName( ui.OPT_3D_CAPACITY );
    ui.desktop.cbShow3dCapacity.addItemListener( (ItemListener)ui );
    panel.add(ui.desktop.cbShow3dCapacity);

    
    getContentPane().add(panel, BorderLayout.CENTER);
  }
	
}

//NetVisUIChildFrame3 Klasse fr Grafikfenster
class NetVisUIChildFrame3 extends JInternalFrame
{
  public NetVisUIChildFrame3( String title, NetVis nv, EventListener listener )
  {
    super(title, true, true);
    setIconifiable(false);
    setMaximizable(false);
    setClosable(false);
    setResizable(true);
    setBackground(Color.white);
    getContentPane().add(nv, BorderLayout.CENTER);

    JButton btn = new JButton( "Reset" );
    btn.addActionListener( (ActionListener)listener );
    
    getContentPane().add( btn, BorderLayout.SOUTH );
  }
}
